@@ -28,6 +28,9 @@
 #' @param time_type If "auto", `periods` are used for
 #' the time expansion when days, weeks, months or years are specified,
 #' and `durations` are used otherwise.
+#' @param shift Value used to shift the time IDs. Typically this is 1 to ensure the
+#' IDs start at 1 but can be 0 or even negative if for example
+#' your time values are going backwards in time.
 #'
 #' @returns
 #' An integer vector the same length as `x`.
@@ -50,16 +53,22 @@
 #'
 #' @export
 time_id <- function(x, time_by = NULL, g = NULL, na_skip = TRUE,
-                    time_type = c("auto", "duration", "period")){
-  out <- time_elapsed(x, time_by = time_by, g = g,
-                      na_skip = na_skip,
-                      time_type = time_type,
-                      rolling = FALSE)
-  out_rounded <- round(out)
+                    time_type = c("auto", "duration", "period"),
+                    shift = 1L){
+  check_is_time_or_num(x)
+  time_by <- time_by_get(x, time_by)
+  check_length(shift, 1)
+  elapsed <- time_elapsed(x, time_by = time_by, g = g,
+                          na_skip = na_skip,
+                          time_type = time_type,
+                          rolling = FALSE)
   # Make sure we don't lose precision when converting to integer
-  which_whole_num <- which(rel_diff(out, out_rounded) < sqrt(.Machine$double.eps))
-  out[which_whole_num]  <- out_rounded[which_whole_num]
-  as.integer(out) + 1L
+  out <- trunc(elapsed)
+  elapsed_rounded <- round(elapsed)
+  elapsed_truncated <- which(double_equal(elapsed, elapsed_rounded) & (elapsed_rounded != out))
+  out[elapsed_truncated] <- elapsed_rounded[elapsed_truncated]
+  as.integer(out) + shift
+  # as.integer(elapsed + 1e-10) + shift
 }
 # To more closely match collapse::timeid one can use the below 3 lines
 # time_diff_gcd <- time_diff_gcd(x)
