@@ -226,7 +226,7 @@ time_roll_sum <- function(x, window = Inf,
                          gx = sorted_g,
                          gbreaks = sorted_g,
                          right = close_left_boundary)
-  adj_window[is.na(adj_window)] <- 0L
+  adj_window[collapse::whichNA(adj_window)] <- 0L
   final_window <- naive_window - adj_window
   # if (lag != 0){
   #   x <- flag2(x, n = lag, g = sorted_g)
@@ -250,7 +250,7 @@ time_roll_sum <- function(x, window = Inf,
   # For duplicate times, we take the last value of each duplicate
   out <- glast(out, g = sorted_g2)
   if (!groups_are_sorted){
-    out <- collapse::greorder(out, g = g2)
+    out <- greorder2(out, g = g2)
   }
   out
 }
@@ -328,7 +328,7 @@ time_roll_mean <- function(x, window = Inf,
                          gx = sorted_g,
                          gbreaks = sorted_g,
                          right = close_left_boundary)
-  adj_window[is.na(adj_window)] <- 0L
+  adj_window[collapse::whichNA(adj_window)] <- 0L
   final_window <- naive_window - adj_window
   # if (lag != 0){
   #   x <- flag2(x, n = lag, g = sorted_g)
@@ -352,7 +352,7 @@ time_roll_mean <- function(x, window = Inf,
   # For duplicate times, we take the last mean value of each duplicate
   out <- glast(out, g = sorted_g2)
   if (!groups_are_sorted){
-    out <- collapse::greorder(out, g = g2)
+    out <- greorder2(out, g = g2)
   }
   out
 }
@@ -503,170 +503,7 @@ time_roll_growth_rate <- function(x, window = Inf,
   }
   out
 }
-# Using a rolling-joing method
-# time_roll_sum2 <- function(x, window,
-#                            time = seq_along(x),
-#                            weights = NULL,
-#                            g = NULL,
-#                            partial = TRUE,
-#                            close_left_boundary = FALSE,
-#                            na.rm = TRUE,
-#                            time_type = c("auto", "duration", "period"),
-#                            roll_month = "preday", roll_dst = "pre",
-#                            ...){
-#   check_index_not_missing(time)
-#   window <- time_by_get(time, time_by = window)
-#   time_num <- time_by_num(window)
-#   time_unit <- time_by_unit(window)
-#   time_subtract <- add_names(list(-time_num), time_unit)
-#   window_size <- time_num
-#   if (length(window_size) != 1L){
-#     stop("time window size must be of length 1")
-#   }
-#   has_groups <- !is.null(g)
-#   check_is_time_or_num(time)
-#   g <- GRP2(g, return.groups = FALSE, return.order = FALSE)
-#   group_id <- group_id(g)
-#   if (has_groups){
-#     group_sizes <- GRP_group_sizes(g)
-#     n_groups <- GRP_n_groups(g)
-#     g2 <- GRP2(list(group_id, time), return.groups = FALSE)
-#   } else {
-#     g2 <- GRP2(time, return.groups = FALSE)
-#     group_sizes <- length(x)
-#     group_id <- collapse::alloc(1L, length(x))
-#     n_groups <- min(1L, length(x))
-#   }
-#   groups_are_sorted <- GRP_is_sorted(g2)
-#   group_id2 <- GRP_group_id(g2)
-#   if (!groups_are_sorted){
-#     group_order <- GRP_order(g2)
-#     x <- x[group_order]
-#     time <- time[group_order]
-#     group_id <- group_id[group_order]
-#     group_id2 <- group_id2[group_order]
-#   }
-#   time_start <- time_add2(time, time_by = time_subtract,
-#                           roll_month = roll_month,
-#                           roll_dst = roll_dst)
-#   naive_window <- sequence(group_sizes)
-#   dt1 <- collapse::qDT(list(group_id = group_id,
-#                             time = time))
-#   dt2 <- collapse::qDT(list(group_id = group_id,
-#                             time_start = time_start))
-#   data.table::setattr(dt1, "sorted", c("group_id", "time"))
-#   data.table::setattr(dt2, "sorted", c("group_id", "time_start"))
-#   if (close_left_boundary){
-#     naive_window2 <- dt1[dt2, on = list(group_id, time < time_start),
-#                          which = TRUE, mult = "last"]
-#   } else {
-#     naive_window2 <- dt1[dt2, on = list(group_id, time <= time_start),
-#                          which = TRUE, mult = "last"]
-#   }
-#   naive_window2 <- seq_along(group_id) - naive_window2
-#   final_window <- data.table::fcoalesce(naive_window2, naive_window)
-#   out <- frollsum3(x, n = final_window,
-#                    weights = weights,
-#                    adaptive = TRUE, align = "right",
-#                    na.rm = na.rm, ...)
-#   if (!partial){
-#     elapsed <- time_elapsed(time, time_by = window, g = group_id,
-#                             rolling = FALSE)
-#     out[double_lt(elapsed, 1)] <- NA_real_
-#
-#   }
-#   # For duplicate times, we take the last mean value of each duplicate
-#   out <- glast(out, g = group_id2)
-#   if (!groups_are_sorted){
-#     out <- collapse::greorder(out, g = g2)
-#   }
-#   out
-# }
-# Using a rolling-join method
-# time_roll_mean2 <- function(x, window,
-#                            time = NULL,
-#                            weights = NULL,
-#                            g = NULL,
-#                            partial = TRUE,
-#                            close_left_boundary = FALSE,
-#                            na.rm = TRUE,
-#                            time_type = c("auto", "duration", "period"),
-#                            roll_month = "preday", roll_dst = "pre",
-#                            ...){
-#   if (is.null(time)){
-#     return(roll_sum(x, window = window,
-#                     g = g, weights = weights,
-#                     partial = partial,
-#                     na.rm = na.rm, ...))
-#   }
-#   check_index_not_missing(time)
-#   window <- time_by_get(time, time_by = window)
-#   time_num <- time_by_num(window)
-#   time_unit <- time_by_unit(window)
-#   time_subtract <- add_names(list(-time_num), time_unit)
-#   window_size <- time_num
-#   if (length(window_size) != 1L){
-#     stop("time window size must be of length 1")
-#   }
-#   has_groups <- !is.null(g)
-#   check_is_time_or_num(time)
-#   g <- GRP2(g, return.groups = FALSE, return.order = FALSE)
-#   group_id <- group_id(g)
-#   if (has_groups){
-#     group_sizes <- GRP_group_sizes(g)
-#     n_groups <- GRP_n_groups(g)
-#     g2 <- GRP2(list(group_id, time), return.groups = FALSE)
-#   } else {
-#     g2 <- GRP2(time, return.groups = FALSE)
-#     group_sizes <- length(x)
-#     group_id <- collapse::alloc(1L, length(x))
-#     n_groups <- min(1L, length(x))
-#   }
-#   groups_are_sorted <- GRP_is_sorted(g2)
-#   group_id2 <- GRP_group_id(g2)
-#   if (!groups_are_sorted){
-#     group_order <- GRP_order(g2)
-#     x <- x[group_order]
-#     time <- time[group_order]
-#     group_id <- group_id[group_order]
-#     group_id2 <- group_id2[group_order]
-#   }
-#   time_start <- time_add2(time, time_by = time_subtract,
-#                           roll_month = roll_month,
-#                           roll_dst = roll_dst)
-#   naive_window <- sequence(group_sizes)
-#   dt1 <- collapse::qDT(list(group_id = group_id,
-#                             time = time))
-#   dt2 <- collapse::qDT(list(group_id = group_id,
-#                             time_start = time_start))
-#   data.table::setattr(dt1, "sorted", c("group_id", "time"))
-#   data.table::setattr(dt2, "sorted", c("group_id", "time_start"))
-#   if (close_left_boundary){
-#     naive_window2 <- dt1[dt2, on = list(group_id, time < time_start),
-#                          which = TRUE, mult = "last"]
-#   } else {
-#     naive_window2 <- dt1[dt2, on = list(group_id, time <= time_start),
-#                          which = TRUE, mult = "last"]
-#   }
-#   naive_window2 <- seq_along(group_id) - naive_window2
-#   final_window <- data.table::fcoalesce(naive_window2, naive_window)
-#   out <- frollmean3(x, n = final_window,
-#                     weights = weights,
-#                     adaptive = TRUE, align = "right",
-#                     na.rm = na.rm, ...)
-#   if (!partial){
-#     elapsed <- time_elapsed(time, time_by = window, g = group_id,
-#                             rolling = FALSE)
-#     out[double_lt(elapsed, 1)] <- NA_real_
-#
-#   }
-#   # For duplicate times, we take the last mean value of each duplicate
-#   out <- glast(out, g = group_id2)
-#   if (!groups_are_sorted){
-#     out <- collapse::greorder(out, g = g2)
-#   }
-#   out
-# }
+
 #' @rdname time_roll
 #' @export
 time_roll_window_size <- function(time, window = Inf,
@@ -833,187 +670,114 @@ time_roll_apply <- function(x, window = Inf, fun,
   # out <- glast(out, g = group_id)
   # out
 }
-# Working alternative
-# time_roll_sum2 <- function(x, window,
-#                           time = NULL,
-#                           weights = NULL,
-#                           g = NULL,
-#                           partial = TRUE,
-#                           close_left_boundary = FALSE,
-#                           na.rm = TRUE,
-#                           time_type = c("auto", "duration", "period"),
-#                           roll_month = "preday", roll_dst = "pre",
-#                           ...){
-#   if (is.null(time)){
-#     return(roll_sum(x, window = window,
-#                     g = g, weights = weights,
-#                     partial = partial,
-#                     na.rm = na.rm, ...))
-#   }
-#   if (anyNA(time)){
-#     stop("time index must not contain NA values")
-#   }
-#   window <- time_by_get(time, time_by = window)
-#   time_num <- time_by_num(window)
-#   time_unit <- time_by_unit(window)
-#   time_subtract <- add_names(list(-time_num), time_unit)
-#   window_size <- time_num
-#   if (length(window_size) != 1L){
-#     stop("time window size must be of length 1")
-#   }
-#   has_groups <- !is.null(g)
-#   check_is_time_or_num(time)
-#   g <- GRP2(g, return.groups = FALSE, return.order = FALSE)
-#   group_id <- group_id(g)
-#   if (has_groups){
-#     group_sizes <- GRP_group_sizes(g)
-#     n_groups <- GRP_n_groups(g)
-#     g2 <- GRP2(list(group_id, time), return.groups = FALSE)
-#   } else {
-#     g2 <- GRP2(time, return.groups = FALSE)
-#     group_sizes <- length(x)
-#     n_groups <- min(1L, length(x))
-#   }
-#   groups_are_sorted <- GRP_is_sorted(g2)
-#   group_id2 <- GRP_group_id(g2)
-#   if (!groups_are_sorted){
-#     group_order <- GRP_order(g2)
-#     x <- x[group_order]
-#     time <- time[group_order]
-#     group_id <- group_id[group_order]
-#     group_id2 <- group_id2[group_order]
-#   }
-#   time_start <- time_add2(time, time_by = time_subtract,
-#                           roll_month = roll_month,
-#                           roll_dst = roll_dst)
-#   naive_window <- sequence(group_sizes)
-#   if (has_groups){
-#     dt <- data.table::data.table(time_start = time_as_number(time_start),
-#                                  time = time_as_number(time),
-#                                  group_id = group_id)
-#     data.table::setattr(dt, "sorted", "group_id")
-#     dt[, ("adj_window") := .bincode(get("time_start"), get("time"),
-#                                     right = close_left_boundary,
-#                                     include.lowest = FALSE),
-#        by = "group_id"]
-#     adj_window <- dt[["adj_window"]]
-#     adj_window[is.na(adj_window)] <- 0L
-#   } else {
-#     adj_window <- findInterval(time_start, time, left.open = close_left_boundary)
-#     # adj_window <- .bincode(as.double(unclass(time_start)),
-#     #                        as.double(unclass(time)),
-#     #                        right = close_left_boundary,
-#     #                        include.lowest = FALSE)
-#     # adj_window[is.na(adj_window)] <- 0L
-#   }
-#   final_window <- naive_window - adj_window
-#   out <- frollsum3(x, n = final_window,
-#                     weights = weights,
-#                     adaptive = TRUE, align = "right",
-#                     na.rm = na.rm, ...)
-#   if (!partial){
-#     elapsed <- time_elapsed(time, time_by = window, g = group_id,
-#                             rolling = FALSE)
-#     out[double_lt(elapsed, 1)] <- NA_real_
-#
-#   }
-#   # For duplicate times, we take the last mean value of each duplicate
-#   out <- glast(out, g = group_id2)
-#   if (!groups_are_sorted){
-#     out <- collapse::greorder(out, g = g2)
-#   }
-#   out
-# }
-# Working alternative
-# time_roll_mean3 <- function(x, window,
-#                            time = NULL, g = NULL,
-#                            weights = NULL,
-#                            partial = TRUE,
-#                            close_left_boundary = FALSE,
-#                            na.rm = TRUE,
-#                            time_type = c("auto", "duration", "period"),
-#                            roll_month = "preday", roll_dst = "pre",
-#                            ...){
-#   if (is.null(time)){
-#     return(roll_mean(x, window = window,
-#                      g = g, weights = weights,
-#                      partial = partial,
-#                      na.rm = na.rm, ...))
-#   }
-#   if (anyNA(time)){
-#     stop("time index must not contain NA values")
-#   }
-#   window <- time_by_get(time, time_by = window)
-#   time_num <- time_by_num(window)
-#   time_unit <- time_by_unit(window)
-#   time_subtract <- add_names(list(-time_num), time_unit)
-#   window_size <- time_num
-#   if (length(window_size) != 1L){
-#     stop("time window size must be of length 1")
-#   }
-#   has_groups <- !is.null(g)
-#   check_is_time_or_num(time)
-#   g <- GRP2(g, return.groups = FALSE, return.order = FALSE)
-#   group_id <- group_id(g)
-#   if (has_groups){
-#     group_sizes <- GRP_group_sizes(g)
-#     n_groups <- GRP_n_groups(g)
-#     g2 <- GRP2(list(group_id, time), return.groups = FALSE)
-#   } else {
-#     g2 <- GRP2(time, return.groups = FALSE)
-#     group_sizes <- length(x)
-#     n_groups <- min(1L, length(x))
-#   }
-#   groups_are_sorted <- GRP_is_sorted(g2)
-#   group_id2 <- GRP_group_id(g2)
-#   if (!groups_are_sorted){
-#     group_order <- GRP_order(g2)
-#     x <- x[group_order]
-#     time <- time[group_order]
-#     group_id <- group_id[group_order]
-#     group_id2 <- group_id2[group_order]
-#   }
-#   time_start <- time_add2(time, time_by = time_subtract,
-#                           roll_month = roll_month,
-#                           roll_dst = roll_dst)
-#   naive_window <- sequence(group_sizes)
-#   if (has_groups){
-#     dt <- data.table::data.table(time_start = time_as_number(time_start),
-#                                  time = time_as_number(time),
-#                                  group_id = group_id)
-#     data.table::setattr(dt, "sorted", "group_id")
-#     # dt[, ("adj_window") := findInterval(get("time_start"), get("time"),
-#     #                                     left.open = close_left_boundary),
-#     #    by = "group_id"]
-#     dt[, ("adj_window") := .bincode(get("time_start"), get("time"),
-#                                     right = close_left_boundary,
-#                                     include.lowest = FALSE),
-#        by = "group_id"]
-#     adj_window <- dt[["adj_window"]]
-#     adj_window[is.na(adj_window)] <- 0L
-#   } else {
-#     adj_window <- findInterval(time_start, time, left.open = close_left_boundary)
-#     # adj_window <- .bincode(as.double(unclass(time_start)),
-#     #                        as.double(unclass(time)),
-#     #                        right = close_left_boundary,
-#     #                        include.lowest = FALSE)
-#     # adj_window[is.na(adj_window)] <- 0L
-#   }
-#   final_window <- naive_window - adj_window
-#   out <- frollmean3(x, n = final_window,
-#                     weights = weights,
-#                     adaptive = TRUE, align = "right",
-#                     na.rm = na.rm, ...)
-#   if (!partial){
-#     elapsed <- time_elapsed(time, time_by = window, g = group_id,
-#                             rolling = FALSE)
-#     out[double_lt(elapsed, 1)] <- NA_real_
-#
-#   }
-#   # For duplicate times, we take the last mean value of each duplicate
-#   out <- glast(out, g = group_id2)
-#   if (!groups_are_sorted){
-#     out <- collapse::greorder(out, g = g2)
-#   }
-#   out
-# }
+
+# Rolling join 2nd version (latest working version)
+time_roll_sum3 <- function(x, window = Inf,
+                          time = seq_along(x),
+                          weights = NULL,
+                          g = NULL,
+                          partial = TRUE,
+                          close_left_boundary = FALSE,
+                          na.rm = TRUE,
+                          time_type = c("auto", "duration", "period"),
+                          roll_month = "preday", roll_dst = "pre",
+                          ...){
+  check_is_time_or_num(time)
+  check_time_not_missing(time)
+  window <- time_by_get(time, time_by = window)
+  time_num <- time_by_num(window)
+  time_unit <- time_by_unit(window)
+  time_subtract <- add_names(list(-time_num), time_unit)
+  unit_time_by <- add_names(list(1), time_unit)
+  window_size <- time_num
+  if (length(window_size) != 1L){
+    stop("time window size must be of length 1")
+  }
+  has_groups <- !is.null(g)
+  g <- GRP2(g, return.groups = FALSE, return.order = TRUE)
+  group_id <- group_id(g)
+  if (has_groups){
+    group_sizes <- GRP_group_sizes(g)
+    n_groups <- GRP_n_groups(g)
+    g2 <- GRP2(list(group_id, time), return.groups = FALSE)
+  } else {
+    g2 <- GRP2(time, return.groups = FALSE)
+    group_sizes <- length(x)
+    n_groups <- min(1L, length(x))
+  }
+  groups_are_sorted <- GRP_is_sorted(g2)
+  group_id2 <- GRP_group_id(g2)
+  if (!groups_are_sorted){
+    group_order <- GRP_order(g2)
+    sorted_df <- new_tbl(x = x, time = time,
+                         group_id = group_id,
+                         group_id2 = group_id2,
+                         weights = weights) %>%
+      df_row_slice(group_order)
+    x <- .subset2(sorted_df, "x")
+    time <- .subset2(sorted_df, "time")
+    group_id <- .subset2(sorted_df, "group_id")
+    group_id2 <- .subset2(sorted_df, "group_id2")
+    weights <- fpluck(sorted_df, "weights")
+  }
+  sorted_g2 <- sorted_group_id_to_GRP(group_id2,
+                                      n_groups = GRP_n_groups(g2),
+                                      group_sizes = GRP_group_sizes(g2),
+                                      group.starts = FALSE)
+  if (has_groups){
+    sorted_g <- sorted_group_id_to_GRP(group_id,
+                                       n_groups = n_groups,
+                                       group_sizes = group_sizes)
+  } else {
+    sorted_g <- NULL
+  }
+  time_start <- time_add2(time, time_by = time_subtract,
+                          roll_month = roll_month,
+                          roll_dst = roll_dst)
+  naive_window <- sequence(group_sizes)
+  dt1 <- collapse::qDT(list3(group_id = group_id,
+                             time = time))
+  dt2 <- collapse::qDT(list3(group_id = group_id,
+                             time_start = time_start))
+  data.table::setattr(dt1, "sorted", names(dt1))
+  data.table::setattr(dt2, "sorted", names(dt2))
+  if (close_left_boundary){
+    if (has_groups){
+      naive_window2 <- dt1[dt2, on = list(group_id, time < time_start),
+                           which = TRUE, mult = "last"]
+    } else {
+      naive_window2 <- dt1[dt2, on = list(time < time_start),
+                           which = TRUE, mult = "last"]
+    }
+  } else {
+    if (has_groups){
+      naive_window2 <- dt1[dt2, on = list(group_id, time <= time_start),
+                           which = TRUE, mult = "last"]
+    } else {
+      naive_window2 <- dt1[dt2, on = list(time <= time_start),
+                           which = TRUE, mult = "last"]
+    }
+  }
+  naive_window2 <- seq_along(x) - naive_window2
+  final_window <- data.table::fcoalesce(naive_window2, naive_window)
+  out <- frollsum3(x, n = final_window,
+                   weights = weights,
+                   adaptive = TRUE, align = "right",
+                   na.rm = na.rm, ...)
+  if (!partial){
+    elapsed <- time_elapsed(time, time_by = unit_time_by,
+                            g = sorted_g, rolling = FALSE)
+    if (close_left_boundary){
+      is_partial <- double_lte(abs(elapsed) + 1, time_num)
+    } else {
+      is_partial <- double_lt(abs(elapsed) + 1, time_num)
+    }
+    out[is_partial] <- NA_real_
+  }
+  # For duplicate times, we take the last value of each duplicate
+  out <- glast(out, g = sorted_g2)
+  if (!groups_are_sorted){
+    out <- collapse::greorder(out, g = g2)
+  }
+  out
+}
